/*
 * Copyright (c) 2005, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ListAttribute.java 2351 2006-02-25 10:55:57Z jmettraux $
 */

//
// ListAttribute.java
//
// john.mettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 31.10.2002 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.engine.workitem;

/**
 * no comment
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-02-25 11:55:57 +0100 (Sat, 25 Feb 2006) $
 * <br>$Id: ListAttribute.java 2351 2006-02-25 10:55:57Z jmettraux $ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public class ListAttribute

    extends CollectionAttribute

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(ListAttribute.class.getName());

    //
    // FIELDS

    protected java.util.ArrayList list = null;

    //
    // CONSTRUCTORS

    public ListAttribute ()
    {
        super();

        this.list = new java.util.ArrayList();
    }

    public ListAttribute (final int initialSize)
    {
        super();

        this.list = new java.util.ArrayList(initialSize);
    }

    //
    // METHODS

    public void add (Attribute a)
    {
        this.list.add(a);
    }

    public void add (int index, Attribute a)
    {
        this.list.add(index, a);
    }

    public int indexOf (Attribute a)
    {
        return this.list.indexOf(a);
    }

    public Attribute get (final int index)
    {
        //log.debug("get() i = "+index);

        int i = index;

        if (i < 0) i = this.list.size() + i;

        //log.debug("get() i = "+i);

        if (i >= this.list.size()) return null;

        return (Attribute)this.list.get(i);
    }

    /**
     * Meant for get("3") or get("0"). If the key doesn't parse to an
     * integer, null will be returned.
     */
    public Attribute get (final String key)
    {
        if (key == null) return null;

        //
        // is it a 'range get' ?

        final int icolon = key.indexOf(":");

        if (icolon > -1)
        {
            if (icolon != key.lastIndexOf(":")) return null;

            try
            {
                final ListAttribute la = new ListAttribute();
                
                final String[] ss = key.split(":");

                final int start = Integer.parseInt(ss[0]);
                final int end = Integer.parseInt(ss[1]);

                int inc = 1; 
                if (end < start) inc = -1;

                for (int i = start; ; i = i + inc)
                {
                    if (i > end && inc > 0) break;
                    if (i < end && inc < 0) break;

                    final Attribute a = get(i);

                    if (a != null) la.add(a);
                }

                return la;
            }
            catch (final Throwable t)
            {
                log.debug("get() failure", t);

                // ignore and return null
            }
            return null;
        }

        //
        // 'set of indexes' get ?

        if (key.indexOf(",") > -1)
        {
            final String[] ss = key.split(",");

            final ListAttribute la = new ListAttribute();

            for (int i=0; i<ss.length; i++)
            {
                try
                {
                    la.add(this.get(Integer.parseInt(ss[i])));
                }
                catch (final Throwable t)
                {
                    // ignore : nothing gets thus added
                }
            }

            return la;
        }

        //
        // regular (one index) get

        try
        {
            return get(Integer.parseInt(key));
        }
        catch (final Throwable t)
        {
            // meant for intercepting NumberFormatExceptions
            // and ArrayIndexOutOfBoundExceptions.
        }
        return null;
    }

    /**
     * This method is used by setField in CollectionAttribute,
     * it has thus an internal value rather than an API value.
     */
    public void cput (final String key, final Attribute value)
    {
        try
        {
            final int i = Integer.parseInt(key);
            this.list.set(i, value);
        }
        catch (final Throwable t)
        {
            log.warn("cput() failed to set key '"+key+"'", t);
        }
    }

    public void remove (Attribute a)
    {
        this.list.remove(a);
    }

    public int size ()
    {
        return this.list.size();
    }

    public java.util.Iterator iterator ()
    {
        return this.list.iterator();
    }

    public void clear ()
    {
        this.list.clear();
    }

    public int hashCode ()
    {
        return this.list.hashCode();
    }

    public Object clone ()
    {
        ListAttribute clone = new ListAttribute(this.size());

        java.util.Iterator it = this.list.iterator();
        while (it.hasNext())
        {
            Attribute a = (Attribute)it.next();
            clone.add((Attribute)a.clone());
        }

        return clone;
    }

    /**
     * Returns a String of the contained values, separated by commas.
     */
    public String toString ()
    {
        final StringBuffer sb = new StringBuffer();

        //sb.append("[ ");

        final java.util.Iterator it = this.iterator();
        while (it.hasNext())
        {
            sb.append(it.next().toString());

            if (it.hasNext()) sb.append(", ");
        }

        //sb.append(" ]");

        return sb.toString();
    }

}
